clear all
close all
clc

K = 25; % # of states
maxIter = 100; % maximum number of allowed iterations
tol = 10^-3;   % tolerance for convergence
no_repetitions = 1; % Run vbhmm 100 times and choose the best model
init_option = 3; %1-task, 2-random, 3-kmeans

%%%%% simulate data %%%%%%%%
no_subjs = 20;
len = 200; % #of time samples
transProb = [0.8 0.2;0.15 0.85];
pio = [0.5,0.5];
mus = zeros(2);
Covs(:,:,1) = [1 -1;-1,1];
Covs(:,:,2) = [1 0;0 1];

%%%%%%%%%%%%%%% Simulate Data from a Non-HMM Model %%%%%%%
load('stim_design.mat')
std_noise = 0.01;
for n = 1:no_subjs
    [data1,task1] = sim_corr_data(stim_design,std_noise,mus,Covs);
    data{n} = data1;
    actual_states{n} = task1;
end
task = [];

%%%%%%%%%%%%%%%%%%%%%Run HMM  %%%%%%%%%%%%%%%%%
[Model]= vb_cont_hmm_multi_subj(data,K,maxIter,tol,no_repetitions,task,init_option);
%load 2_node_results_5subjs.mat
%matlabpool close
%%%%%%%% Choose the Model that has the highest evidence %%%%%%%%%%%
for repetition = 1:length(Model)
    F = Model{repetition}.F;
    if repetition == 1
        model = Model{repetition};
    elseif (max(model.F) < max(F))
        model = Model{repetition};
    end
end
no_subjs = length(model.state_prob_smooth);
no_obs = size(model.state_prob_smooth{1},2);
[est_states] =  est_states_vitterbi(data,model);
est_states = reshape(est_states',[no_obs,no_subjs]);
state_prob_smooth = [];
for n = 1:no_subjs
    state_prob_smooth = [state_prob_smooth model.state_prob_smooth{n}];
end
[max_probs,post_states] = max(state_prob_smooth);
post_states = reshape(post_states',[no_obs,no_subjs]);
st = 1;
for subj = 1:no_subjs
    state_prob_smooth_subj = state_prob_smooth(:,st:st+no_obs-1);
    figure(subj)
    subplot(611)
    imagesc(state_prob_smooth_subj)
    title('State Probability Matrix')
    subplot(612)
    plot(actual_states{subj},'k','linewidth',2)
    title('Actual Task')
    subplot(613)
    plot(est_states(:,subj),'r--','linewidth',2)
    title('Vitterbi Path')
    subplot(614)
    plot(post_states(:,subj),'g','linewidth',2)
    title('state(t)) = arg-max(state_prob_matrix(:,t)')
    counts_vitterbi = zeros(1,K);
    counts_post = zeros(1,K);
    for k = 1:K
        counts_vitterbi(k) = length(find(est_states(:,subj) == k));
        counts_post(k) = length(find(post_states(:,subj) == k));
    end
    subplot(615)
    stem(counts_vitterbi)
    title('Percent Occurence of each etate (Vitterbi Path)')
    subplot(616)
    stem(counts_post)
    title('Percent Occurence of each state (Posterior Prob) ')
    st = st + no_obs;
end
figure,
for k = 1:length(Model)
    plot(Model{k}.F,'o-')
    hold on
    title('Log-Lower Bound')
end
plot(model.F,'or-')

for k = 1:K
    ap = model.ap(k); bp = model.bp(:,:,k);
    est_cov(:,:,k) = bp/ap;
    invD = inv(diag(sqrt(diag(est_cov(:,:,k)))));
    pearson_corr(:,:,k) = invD*est_cov(:,:,k)*invD;
end
%estimated state transition matrix A
Wa = model.Wa;
Wa = Wa';
H = size(Wa,1);
Aest = Wa./repmat(sum(Wa,2),1,H);% transition distribution p(h(t)|h(t-1))
%estimeted pi
Wpi = model.Wpi;
piest = Wpi./sum(Wpi);

for k = 1:K
    counts_post(k) = length(find(est_states == k));
end
counts_post = counts_post/sum(counts_post);
[percent_dominant dominant_states] = sort(counts_post,'descend');
no_dom_states = 1;





