function S = mc_sample(prior, trans, len, numex)
% Generate random sequences from a Markov chain
% STATE = SAMPLE_MC(PRIOR, TRANS, LEN) generates a sequence of length LEN.
%
% STATE = SAMPLE_MC(PRIOR, TRANS, LEN, N) generates N rows each of length LEN.

% This file is from pmtk3.googlecode.com


if nargin==3
  numex = 1;
end

S = zeros(numex,len);
for i=1:numex
  S(i, 1) = sampleDiscrete(prior);
  for t=2:len
    S(i, t) = sampleDiscrete(trans(S(i,t-1),:));
  end
end

end




function M = sampleDiscrete(prob, r, c)
% Like the built in 'rand', except we draw from a non-uniform discrete distrib
% M = sample_discrete(prob, r, c)
%
% Example: sample_discrete([0.8 0.2], 1, 10) generates a row vector of 10 random integers from {1,2},
% where the prob. of being 1 is 0.8 and the prob of being 2 is 0.2.

% This file is from pmtk3.googlecode.com

n = length(prob);

if nargin == 1
    r = 1; c = 1;
elseif nargin == 2
    c = r;
end

R = rand(r, c);
M = ones(r, c);
cumprob = cumsum(prob(:));

if n < r*c
    for i = 1:n-1
        M = M + (R > cumprob(i));
    end
else
    % loop over the smaller index - can be much faster if length(prob) >> r*c
    cumprob2 = cumprob(1:end-1);
    for i=1:r
        for j=1:c
            M(i,j) = sum(R(i,j) > cumprob2)+1;
        end
    end
end


% Slower, even though vectorized
%cumprob = reshape(cumsum([0 prob(1:end-1)]), [1 1 n]);
%M = sum(R(:,:,ones(n,1)) > cumprob(ones(r,1),ones(c,1),:), 3);

% convert using a binning algorithm
%M=bindex(R,cumprob);

end
