function newAtrans = mergeStates(Atrans,merged_states)

%The merged state is always considered as the first state in the newAtrans
%The other (unmerged states) are ordered according to their previous order.

%Example: Let there be four state: [1, 2, 3, 4]. Let the states to be merged
% are 2 and 3. Then the new states are ordered as : State 1 -  Old State 2 union 3,
%                                                   State 2 = Old state 1
%                                                   State 3 = Old State 4


[rows,cols] = size(Atrans);

% Stationary Distribution  (Marginal Probabilities)
pis = mcStatDist(Atrans,2); %Using method # 2

%Joint Distribution :
Ajoint = diag(pis) * Atrans;

%%%%%%%%%%%%%%%%%%  New Joint Distribution %%%%%%%%
newAjoint = zeros(rows-1,cols-1);
%Transition to within the merged state
sum1 = 0;
for r = 1:length(merged_states)
    i1 = merged_states(r);
    for c = 1:length(merged_states)
        i2 = merged_states(c);
        sum1 = sum1 + Ajoint(i1,i2);
    end
end
newAjoint(1,1) = sum1; % Always consider the merged state as 1 in newAjoint

%Transition from the merged state to the other states
other_states = 1:rows; other_states(merged_states) = [];
ix = 2; %starting location for the other state
for c = 1:length(other_states)
    i2 = other_states(c);
    sum1 = 0;
    for r = 1:length(merged_states)
        i1 = merged_states(r);
        sum1 = sum1 + Ajoint(i1,i2);
    end
    newAjoint(1,ix) = sum1;
    ix = ix + 1;    %pointer to the next other state
end

%Transition to the merged state from the other states
ix = 2; %starting location for the other state
for c = 1:length(other_states)
    i2 = other_states(c);
    sum1 = 0;
    for r = 1:length(merged_states)
        i1 = merged_states(r);
        sum1 = sum1 + Ajoint(i2,i1);
    end
    newAjoint(ix,1) = sum1;
    ix = ix + 1;    %pointer to the next other state
end

% Transitions from the other state to the other states
ix1 = 2;
for r = 1:length(other_states)
    i1 = other_states(r);
    ix2 = 2;
    for c = 1:length(other_states)
        i2 = other_states(c);
        newAjoint(ix1,ix2) = Ajoint(i1,i2);
        ix2 = ix2 + 1;
    end
    ix1 = ix1 + 1;
end

%Compute Conditional distribution
marginal_dist = sum(newAjoint,2);
newAtrans = diag(1./marginal_dist) * newAjoint;
